@@ -6,7 +6,7 @@ import time
 import functools
 import jax.numpy as jnp
 import scipy.sparse as ssp
-from scipy import ndarray
+from numpy import ndarray, isnan
 from .plots import spy
 from jax.experimental.host_callback import id_print as jax_print
 # from jaxlib.xla_extension import DeviceArray
@@ -32,7 +32,7 @@ def value_and_jac(f, sparse=False):
     return lambda x: value_and_jac_inner(f, x, sparse)
 
 
-def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=None, func_returns_jac=False, inspect_jac=False, verbose=False, verbose_jac_det=False):
+def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=None, func_returns_jac=False, inspect_jac=False, verbose=False, verbose_jac=False):
     """Newton method for root finding using automatic differenciation with jax. The argument `func` must be jittable with jax.
 
     ...
@@ -59,7 +59,7 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
         If `True`, use grgrlib.plots.spy to visualize the jacobian
     verbose : bool, optional
         Whether to display messages
-    verbose_jac_det : bool, optional
+    verbose_jac : bool, optional
         Whether to supply additional information on the determinant of the jacobian (computationally more costly).
 
     Returns
@@ -69,8 +69,8 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
     """
 
     st = time.time()
-    verbose_jac_det |= inspect_jac
-    verbose |= verbose_jac_det
+    verbose_jac |= inspect_jac
+    verbose |= verbose_jac
 
     if jac is None and not func_returns_jac:
         if sparse:
@@ -98,6 +98,13 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
         else:
             fval, jacval = func(xi), jac(xi)
 
+        jac_is_nan = isnan(jacval.data) if isinstance(
+            jacval, ssp._arrays.csr_array) else jnp.isnan(jacval)
+        if jac_is_nan.any():
+            res['success'] = False
+            res['message'] = "The jacobian contains `NaN`s."
+            break
+
         eps_fval = jnp.abs(fval).max()
         if eps_fval < tol:
             res['success'] = True
@@ -110,9 +117,9 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
         if verbose:
             ltime = time.time() - st
             info_str = f'    Iteration {cnt:3d} | max error {eps:.2e} | lapsed {ltime:3.4f}'
-            if verbose_jac_det:
+            if verbose_jac:
                 jacval = jacval.toarray() if sparse else jacval
-                info_str += f' | det {jnp.linalg.det(jacval):1.5g}'
+                info_str += f' | det {jnp.linalg.det(jacval):1.5g} | rank {jnp.linalg.matrix_rank(jacval)}/{jacval.shape[0]}'
                 if inspect_jac:
                     spy(jacval)
 
@@ -129,15 +136,17 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
             break
 
         if jnp.isnan(eps):
-            jacval = jacval.toarray() if not isinstance(jacval, ndarray) else jacval
-            if jnp.isnan(jacval).any():
-                raise Exception(f'The jacobian contains `NaN`s.')
-            else:
-                raise Exception(
-                    f'Newton method returned `NaN` in iter {cnt}. Determinant of jacobian is {jnp.linalg.det(jacval):1.5g}.')
+            res['success'] = False
+            res['message'] = f"Function returns 'NaN's"
+            break
+
+    jacval = jacval.toarray() if isinstance(
+        jacval, ssp._arrays.csr_array) else jacval
 
     res['x'], res['niter'] = xi, cnt
     res['fun'] = func(xi)[0] if func_returns_jac else func(xi)
+    res['jac'] = jacval
+    res['det'] = jnp.linalg.det(jacval)
 
     return res
 
